program elemental
   ! Using MPI module
   use mpi
   ! Using the kernel module ----------------------------------------------
   ! It also loads the wp double precision variable
   use kernels
   !-----------------------------------------------------------------------
   ! Extrae Stuff----------------------------------------------------------
   USE, INTRINSIC :: ISO_C_BINDING, ONLY: C_CHAR, C_NULL_CHAR, C_PTR, C_LOC
   use extrae_module
   !-----------------------------------------------------------------------
   implicit none
   !-----------------------------------------------------------------------
   ! To read array size in command line -----------------------------------
   integer :: cmd_line_size
   integer :: n
   CHARACTER(LEN=100) :: arg
   !-----------------------------------------------------------------------

   ! Variables that can be used in the kernel -----------------------------
   integer :: ii,jj,kk                              ! dummy integers
   real(wp) , allocatable, dimension(:) :: A,B,C,D ! Arrays
   real(wp) :: pa,pb,pc,pd                              ! Parameters
   integer :: array_size = 512
   integer :: ierr
   ! Extrae Stuff ----------------------------------------------------------
   CHARACTER(KIND=C_CHAR,LEN=6), DIMENSION(2), TARGET ::  description_values
   CHARACTER(KIND=C_CHAR,LEN=20) :: evt_desc =  "LoopEvent" !! C_NULL_CHAR
   INTEGER*8, PARAMETER, DIMENSION(2) :: values = (/ 0, 1  /)
   TYPE(C_PTR), DIMENSION(2) :: description_values_ptrs
   description_values(1) = "End  " // C_NULL_CHAR
   description_values_ptrs(1) = C_LOC(description_values(1))
   description_values(2) = "Init" // C_NULL_CHAR
   description_values_ptrs(2) = C_LOC(description_values(2))
   !-----------------------------------------------------------------------
   ! Reading the array size in command line -------------------------------
    n = iargc()
    if (n == 1) then
       call getarg(1,arg) !Grab the 1st command line argument
                          !and store it in the temporary variable
                          !'arg'
       read(arg,*) cmd_line_size   !Now convert string to integer
       array_size = cmd_line_size  !
    end if
   !-----------------------------------------------------------------------
   ! Defining the extrae event --------------------------------------------
   CALL extrae_define_event_type (60000519, evt_desc, 2, values, description_values_ptrs)
   !-----------------------------------------------------------------------
   ! Allocation of arrays with align attribute ----------------------------
   !dir$ attributes align:64 :: A
   !dir$ attributes align:64 :: B
   !dir$ attributes align:64 :: C
   !dir$ attributes align:64 :: D
   allocate(A(array_size))
   allocate(B(array_size))
   allocate(C(array_size))
   allocate(D(array_size))
   !-----------------------------------------------------------------------
   ! Initialization of arrays and parameters ------------------------------
   call random_number(A)
   call random_number(B)
   call random_number(C)
   call random_number(D)
   call random_number(pa)
   call random_number(pb)
   call random_number(pc)
   call random_number(pd)
   !-----------------------------------------------------------------------
   
   ! MPI Initialization and barrier to assert that all threads are running 
   ! at the same timeof arrays and parameters -----------------------------
   call mpi_init(ierr)
   call mpi_barrier(mpi_comm_world,ierr)
   ! Extrae event before the kernel --------------------------------------- 
   CALL extrae_eventandcounters(60000519, 1_8)
   !-----------------------------------------------------------------------
   ! The kernel defined in kernel.f90 is called here ----------------------
   call kernel(A,B,C,D,pa,pb,pc,pd,array_size)
   !-----------------------------------------------------------------------
   ! Extrae event after the kernel ----------------------------------------
   CALL extrae_eventandcounters(60000519, 0_8)
   !-----------------------------------------------------------------------
   ! MPI Barrier ----------------------------------------------------------
   call mpi_barrier(mpi_comm_world,ierr)
   !-----------------------------------------------------------------------
   ! Using the computed variable to avoid compiler optimizations for non
   ! used variables (maybe it is not necessary) --------------------------- 
   do jj=1,array_size
      if (a(jj) < 0 ) then
         a(jj) = 0
         ii = ii + 1
      end if
   end do
   !-----------------------------------------------------------------------
   ! Array deallocation ---------------------------------------------------
   deallocate(A,B,C,D)
   !-----------------------------------------------------------------------
   ! MPI Finalizer --------------------------------------------------------
   call mpi_finalize(ierr)
   !-----------------------------------------------------------------------
end program
